/**
 * flexselect: a jQuery plugin, version: 0.9.0 (2016-09-16)
 * 
 * @requires jQuery v1.3 or later
 * 
 * FlexSelect is a jQuery plugin that makes it easy to convert a select box into a Quicksilver-style, autocompleting, flex matching selection tool.
 * 
 * For usage and examples, visit: http://rmm5t.github.io/jquery-flexselect/
 * 
 * Licensed under the MIT: http://www.opensource.org/licenses/mit-license.php
 * 
 * Copyright (c) 2009-2015, Ryan McGeary (ryan -[at]- mcgeary [*dot*] org)
 */
( function( $ ) {
	$.flexselect = function( select, options ) {
		this.init( select, options );
	};

	$
			.extend(
					$.flexselect.prototype,
					{
						settings : {
							allowMismatch : true,
							allowMismatchBlank : true, // If "true" a user can
							// backspace such that
							// the value is nothing
							// (even if no blank
							// value was provided in
							// the original
							// criteria)
							sortBy : 'score', // 'score' || 'name'
							blankSortBy : 'name', // 'score' || 'name' ||
							// 'initial'
							preSelection : false,
							hideDropdownOnEmptyInput : false,
							selectedClass : "flexselect_selected",
							dropdownClass : "flexselect_dropdown",
							showDisabledOptions : false,
							inputIdTransform : function( id ) {
								return id + "_flexselect";
							},
							inputNameTransform : function( name ) {
								return;
							},
							dropdownIdTransform : function( id ) {
								return id + "_flexselect_dropdown";
							}
						},
						select : null,
						input : null,
						dropdown : null,
						dropdownList : null,
						cache : [],
						results : [],
						lastAbbreviation : null,
						abbreviationBeforeFocus : null,
						selectedIndex : 0,
						picked : false,
						allowMouseMove : true,
						dropdownMouseover : false, // Workaround for poor IE
						// behaviors
						indexOptgroupLabels : false,

						init : function( select, options ) {
							this.settings = $.extend( {}, this.settings,
									options );
							this.select = $( select );
							this.reloadCache();
							this.renderControls();
							this.wire();
						},

						reloadCache : function() {
							var name, group, text, disabled;
							var indexGroup = this.settings.indexOptgroupLabels;
							this.cache = this.select.find( "option" )
									.map(
											function() {
												name = $( this ).text();
												group = $( this ).parent(
														"optgroup" ).attr(
														"label" );
												text = indexGroup ? [ name,
														group ].join( " " )
														: name;
												disabled = $( this ).parent(
														"optgroup" ).attr(
														"disabled" )
														|| $( this ).attr(
																'disabled' );
												return {
													text : $.trim( text ),
													name : $.trim( name ),
													value : $( this ).val(),
													disabled : disabled,
													score : 0.0
												};
											} );
						},

						renderControls : function() {
							var selected = this.settings.preSelection ? this.select
									.find( "option:selected" )
									: null;

							this.input = $(
									"<input type='text' autocomplete='off' />" )
									.attr(
											{
												id : this.settings
														.inputIdTransform( this.select
																.attr( "id" ) ),
												name : this.settings
														.inputNameTransform( this.select
																.attr( "name" ) ),
												accesskey : this.select
														.attr( "accesskey" ),
												tabindex : this.select
														.attr( "tabindex" ),
												style : this.select
														.attr( "style" ),
												placeholder : this.select
														.attr( "data-placeholder" )
											} ).addClass(
											this.select.attr( "class" ) ).val(
											$.trim( selected ? selected.text()
													: '' ) ).css( {
										visibility : 'visible'
									} );

							this.dropdown = $( "<div></div>" )
									.attr(
											{
												id : this.settings
														.dropdownIdTransform( this.select
																.attr( "id" ) )
											} ).addClass(
											this.settings.dropdownClass );
							this.dropdownList = $( "<ul></ul>" );
							this.dropdown.append( this.dropdownList );

							this.select.after( this.input ).hide();
							$( "body" ).append( this.dropdown );
						},

						wire : function() {
							var self = this;

							this.input.click( function() {
								self.lastAbbreviation = null;
								self.focus();
							} );

							this.input.mouseup( function( event ) {
								// This is so Safari selection actually occurs.
								event.preventDefault();
							} );

							this.input
									.focus( function() {
										self.abbreviationBeforeFocus = self.input
												.val();
										self.input[ 0 ].setSelectionRange( 0,
												self.input.val().length );
										// if (!self.picked)
										self.filterResults();
									} );

							this.input
									.blur( function() {
										if ( !self.dropdownMouseover ) {
											self.hide();
											if ( self.settings.allowMismatchBlank
													&& $.trim( $( this ).val() ) == '' ) {
												// self.setValue('');
											}
											else if ( !self.settings.allowMismatch
													&& !self.picked )
												self.reset();
										}
									} );

							this.dropdownList.mouseover( function( event ) {
								if ( !self.allowMouseMove ) {
									self.allowMouseMove = true;
									return;
								}

								if ( event.target.tagName == "LI" ) {
									var rows = self.dropdown.find( "li" );
									self.markSelected( rows
											.index( $( event.target ) ) );
								}
							} );
							this.dropdownList.mouseleave( function() {
								self.markSelected( -1 );
							} );
							this.dropdownList.mouseup( function( event ) {
								self.pickSelected();
								self.focusAndHide();
							} );
							this.dropdownList.bind( "touchstart", function(
									event ) {
								if ( event.target.tagName == "LI" ) {
									var rows = self.dropdown.find( "li" );
									self.markSelected( rows
											.index( $( event.target ) ) );
								}
							} );
							this.dropdown.mouseover( function( event ) {
								self.dropdownMouseover = true;
							} );
							this.dropdown.mouseleave( function( event ) {
								self.dropdownMouseover = false;
							} );
							this.dropdown.mousedown( function( event ) {
								event.preventDefault();
							} );

							this.input.keyup( function( event ) {
								switch ( event.keyCode ) {
								case 13: // return
									event.preventDefault();
									self.pickSelected();
									self.focusAndHide();
									break;
								case 27: // esc
									event.preventDefault();
									self.reset();
									self.focusAndHide();
									break;
								default:
									self.filterResults();
									break;
								}
							} );

							this.input.keydown( function( event ) {
								switch ( event.keyCode ) {
								case 9: // tab
									self.pickSelected();
									self.hide();
									break;
								case 33: // pgup
									event.preventDefault();
									self.markFirst();
									break;
								case 34: // pgedown
									event.preventDefault();
									self.markLast();
									break;
								case 38: // up
									event.preventDefault();
									self.moveSelected( -1 );
									break;
								case 40: // down
									event.preventDefault();
									self.moveSelected( 1 );
									break;
								case 13: // return
								case 27: // esc
									event.preventDefault();
									event.stopPropagation();
									break;
								}
							} );

							var input = this.input;
							this.select.change( function() {
								input.val( $.trim( $( this ).find(
										'option:selected' ).text() ) );
							} );
						},

						filterResults : function() {
							var showDisabled = this.settings.showDisabledOptions;
							var abbreviation = $.trim( this.input.val() );
							var sortByMechanism = ( abbreviation == "" ) ? this.settings.blankSortBy
									: this.settings.sortBy;
							if ( abbreviation == this.lastAbbreviation )
								return;

							// abbreviation changed, call external hook if needed
							if ( typeof handleAbbreviationChange == "function" ) {
								handleAbbreviationChange();
							}

							var results = [];
							$.each( this.cache, function() {
								if ( this.disabled && !showDisabled )
									return;
								this.score = prefixScore( this.text,
										abbreviation );
								if ( this.score > 0.0 )
									results.push( this );
							} );
							this.results = results;

							this.sortResultsBy( sortByMechanism );
							this.renderDropdown();
							this.markFirst();
							this.lastAbbreviation = abbreviation;
							this.picked = false;
							this.allowMouseMove = false;

							if ( this.settings.hideDropdownOnEmptyInput ) {
								if ( abbreviation == "" )
									this.dropdown.hide();
								else
									this.dropdown.show();
							}
						},

						sortResultsBy : function( mechanism ) {
							if ( mechanism == "score" ) {
								this.sortResultsByScore();
							}
							else if ( mechanism == "name" ) {
								this.sortResultsByName();
							}
						},

						sortResultsByScore : function() {
							this.results.sort( function( a, b ) {
								if ( a.score != b.score ) {
									return b.score - a.score;
								}
								else {
									return a.name < b.name ? -1
											: ( a.name > b.name ? 1 : 0 );
								}
							} );
						},

						sortResultsByName : function() {
							this.results.sort( function( a, b ) {
								return a.name < b.name ? -1
										: ( a.name > b.name ? 1 : 0 );
							} );
						},

						renderDropdown : function() {
							var showDisabled = this.settings.showDisabledOptions;
							var dropdownBorderWidth = this.dropdown
									.outerWidth()
									- this.dropdown.innerWidth();
							var inputOffset = this.input.offset();
							this.dropdown
									.css( {
										width : ( this.input.outerWidth() - dropdownBorderWidth )
												+ "px",
										top : ( inputOffset.top + this.input
												.outerHeight() )
												+ "px",
										left : inputOffset.left + "px",
										maxHeight : ''
									} );

							var html = '';
							var disabledAttribute = '';
							$
									.each(
											this.results,
											function() {
												if ( this.disabled
														&& !showDisabled )
													return;
												disabledAttribute = this.disabled ? ' class="disabled"'
														: '';
												html += '<li'
														+ disabledAttribute
														+ '>' + this.name
														+ '</li>';
											} );
							this.dropdownList.html( html );
							this.adjustMaxHeight();
							this.dropdown.show();
						},

						adjustMaxHeight : function() {
							var maxTop = $( window ).height()
									+ $( window ).scrollTop()
									- this.dropdown.outerHeight();
							var top = parseInt( this.dropdown.css( 'top' ), 10 );
							this.dropdown
									.css(
											'max-height',
											top > maxTop ? ( Math
													.max(
															0,
															maxTop
																	- top
																	+ this.dropdown
																			.innerHeight() ) + 'px' )
													: '' );
						},

						markSelected : function( n ) {
							if ( n < 0 || n >= this.results.length )
								return;

							var rows = this.dropdown.find( "li" );
							rows.removeClass( this.settings.selectedClass );

							var row = $( rows[ n ] );
							if ( row.hasClass( 'disabled' ) ) {
								this.selectedIndex = null;
								return;
							}

							this.selectedIndex = n;
							row.addClass( this.settings.selectedClass );
							var top = row.position().top;
							var delta = top + row.outerHeight()
									- this.dropdown.height();
							if ( delta > 0 ) {
								this.allowMouseMove = false;
								this.dropdown.scrollTop( this.dropdown
										.scrollTop()
										+ delta );
							}
							else if ( top < 0 ) {
								this.allowMouseMove = false;
								this.dropdown.scrollTop( Math.max( 0,
										this.dropdown.scrollTop() + top ) );
							}
						},

						pickSelected : function() {
							var selected = this.results[ this.selectedIndex ];
							if ( selected && !selected.disabled ) {
								this.input.val( selected.name );
								this.setValue( selected.value );
								this.picked = true;
							}
							else if ( this.settings.allowMismatch ) {
								this.setValue.val( "" );
							}
							else {
								this.reset();
							}
						},

						setValue : function( val ) {
							//							if ( this.select.val() === val )
							//								return;
							this.select.val( val ).change();
						},

						hide : function() {
							this.dropdown.hide();
							this.lastAbbreviation = null;
						},

						moveSelected : function( n ) {
							this.markSelected( this.selectedIndex + n );
						},
						markFirst : function() {
							this.markSelected( 0 );
						},
						markLast : function() {
							this.markSelected( this.results.length - 1 );
						},
						reset : function() {
							this.input.val( this.abbreviationBeforeFocus );
						},
						focus : function() {
							this.input.focus();
						},
						focusAndHide : function() {
							this.focus();
							this.hide();
						}
					} );

	$.fn.flexselect = function( options ) {
		this
				.each( function() {
					if ( $( this ).data( "flexselect" ) ) {
						$( this ).data( "flexselect" ).reloadCache();
					}
					else if ( this.tagName == "SELECT" ) {
						$( this ).data( "flexselect",
								new $.flexselect( this, options ) );
					}
				} );
		return this;
	};
} )( jQuery );

var useLiquidMetalScoring = false;

function prefixScore( txt, pref ) {
	if ( useLiquidMetalScoring ) {
		return LiquidMetal.score( txt, pref );
	}
	return txt.startsWith( pref ) ? 1.0 : 0.0;
}
