#!/bin/sh
#
# Purpose:
#	moves late objects when it find a moveto flag
#
# Synopsis:
#	moveLateObjects.sh <file> <old_studyUID>
#
# Arguments:
#	file:	the late object
#	old_studyUID:	before the study moved the object belong to this study
#
# Description:
#	First validate the moveto.{old_study_UID} flag file. It has to contain the NewStudyUID,
#	UIDFile, ModFile. Creates a workdirectory. Call CopyDicomFiles on file and move the new 
#	file to it's correct place ($REPOSITORY_DICOM_DATA/xx/yy/zz/$NewStudyUID) and call storescp_reg.sh
#
#  Last Modified by : $Author$
#  Last Modified at : $Date$
#  RCS File Name    : $RCSfile$
#  RCS Revision     : $Revision$
#  Locked by        : $Locker$
#  Symbolic name    : $Name$
#  State            : $State$
#

File=$1
Study=$2

[ -z "$File" ] && exit 1;
[ -z "$Study" ] && exit 1;

[ -z "$IMAGE_TMP" ] && . $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="movelateobjects[$$]"

$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "ARGS: $@"

run_()
{
	$UTILS_HOME/bin/createLockFile "${TMP}/moveto.lock"
	[ $? -ne 0 ] && return 1 # the flag file already exists or could not be created

	# validate the nuke flag file
	if [ ! -f "${TMP}/moveto.${Study}" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "moveto.${Study} moveto file does not exists"
		rm -f ${TMP}/moveto.lock
		return 125
	fi

	. ${TMP}/moveto.${Study}
	
	if [ -z "$NewStudyUID" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "moveto file content is not valid : missing new studyuid"
		rm -f ${TMP}/moveto.lock
		return 1
	fi

	if [ -z "$UIDFile" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "moveto file content is not valid : missing UIDFile"
		rm -f ${TMP}/moveto.lock
		return 1
	fi

	if [ ! -f "${TMP}/moveto.${NewStudyUID}.uids" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "${TMP}/moveto.${NewStudyUID}.uids file does not exist"
		rm -f ${TMP}/moveto.lock
		return 1
	fi

	if [ ! -z "$ModFile" ] && [ ! -f "${TMP}/moveto.${NewStudyUID}.mod" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "${TMP}/moveto.${NewStudyUID}.mod file does not exist"
		rm -f ${TMP}/moveto.lock
		return 1
	fi
		
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "moveto flag file validated: OK"

	# touching the files for safety
	touch "${TMP}/moveto.${Study}"
	touch "${UIDFile}"
	touch "${ModFile}"

	#create work directory
	WDIR=$TMP/moveto.${NewStudyUID}
	if [ -d "$WDIR" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$WDIR working directory exists"
		rm -f ${TMP}/moveto.lock
		return 2
	fi
	mkdir $WDIR

	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "$WDIR working directory created"
	
	#get the new dicom file and copy to it's place to data/dicom and call storescp_reg.sh
	echo "${TMP}/obsolete/${Study}/${File}" > $WDIR/filelist
	$UTILS_HOME/bin/copyDicomFiles -d $WDIR -f $WDIR/filelist -l $UIDFile -m  $ModFile > $WDIR/uidlist

	if [ $? -ne 0 ] ; then
		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "${File}::  $UTILS_HOME/bin/copyDicomFiles -d $WDIR -f $WDIR/filelist -l $UIDFile -m  $ModFile : was not run correctly"
		rm -r $WDIR
		rm -f ${TMP}/moveto.lock
		return 1
	fi

	rm -f $WDIR/filelist
	mv $WDIR/uidlist $UIDFile
	
	NewFile=`ls $WDIR`

	if [ ! -f "$WDIR/$NewFile" ] ; then
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$WDIR/$NewFile file does not exists"
		rm -f ${TMP}/moveto.lock
		return 2
	fi
	
	
	newstudydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $NewStudyUID`
	if [ $? -ne 0 ] ; then
		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Study $NewStudyUID could not be located in repository (maybe the study was removed)"
		rm -r $WDIR
		rm -f ${TMP}/moveto.lock
		return 125 
	fi
	
	tempFile="$IMAGE_TMP/${NewStudyUID}_${NewFile}_$$"
	mv ${WDIR}/${NewFile} "${tempFile}"
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "mv ${WDIR}/${NewFile} ${tempFile}"


	$UTILS_HOME/bin/storeDICOMFile -f "$NewFile" -s "$NewStudyUID" -t "${tempFile}" -D "DELTA_STORESCP_REG_LATE_OBJECT"
	if [ $? -ne 0 ] ; then
		# StorescpReg task could not be placed
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "StorescpReg task cound not be created for $NewStudyUID/$NewFile"
		rm ${tempFile} 2>/dev/null
		rm -r $WDIR
		rm -f ${TMP}/moveto.lock
		return 1
	fi

	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "$WDIR working directory deleted"
	rm -r $WDIR
	rm -f ${TMP}/moveto.lock
	
	return 0

} #run_()

#
# default get resources 
#
get_resources_()
{
	echo "System.DISK"
}


#
# default get display name (for subjob)
#
get_display_name_()
{
	echo "Move late objects received for a merged/split study to result study"
}

get_property_() {
	if [ "$PROPERTY" = "1PST" -a -n "$Study" ] ; then
		echo -n "$Study"
	else
		get_def_property_
	fi
}
