#!/bin/sh
#
# Component: cases
#
# Description:
#   Moves DICOM object to study directory, starts registering
#   the DICOM object into database, and starts autoforward.
#
# Usage:
#   DEPRECATED: use StorescpReg class instead
#   storescp_reg.sh [-e | -r] [-d] [-c] -f <file> -s <study> [-i <sender IP> -a <senderAE>]
#
#   Old form for backward compatibility:
#   storescp_reg.sh [-e | -r] [-d] <file> <study> <sender IP> <senderAE>
#      -e        : called for existing file: reindex only
#      -r        : reindex with autoforward, put to archive, ...
#      -d        : call Dcreg directly instead of via taskd
#      -c        : do an extra cmp check when copying the file
#      file      : File name of the DICOM object
#      study     : Study Instance UID of the DICOM object
#      sender IP : IP address of the DICOM device from where the DICOM object was received
#      sender AE : AE title of the DICOM device from where the DICOM object was received
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

[ -z "$CASES_HOME" ] && . $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="storescpreg[$$]"


INPUT_PARAMS=""

# for identifying legacy parameters
File=""

while getopts 'erdcf:s:i:a:' OPT; do
	case $OPT in
	e) INPUT_PARAMS="$INPUT_PARAMS -e" ;;
	r) INPUT_PARAMS="$INPUT_PARAMS -r" ;;
	d) INPUT_PARAMS="$INPUT_PARAMS -d" ;;
	c) INPUT_PARAMS="$INPUT_PARAMS -c" ;;
	f) File="$OPTARG"; INPUT_PARAMS="$INPUT_PARAMS -f \"$OPTARG\"" ;;
	s) INPUT_PARAMS="$INPUT_PARAMS -s \"$OPTARG\"" ;;
	i) INPUT_PARAMS="$INPUT_PARAMS -a \"$OPTARG\"" ;;
	a) INPUT_PARAMS="$INPUT_PARAMS -i \"$OPTARG\"" ;;
	*) continue ;;
	esac
done

shift $(($OPTIND-1))

if [ -z "$File" ] ; then
	INPUT_PARAMS="$INPUT_PARAMS -l"
	[ -n "$File" ] && INPUT_PARAMS="$INPUT_PARAMS -f \"$File\""
	[ -n "$Study" ] && INPUT_PARAMS="$INPUT_PARAMS -s \"$Study\""
	[ -n "$SenderAddr" ] && INPUT_PARAMS="$INPUT_PARAMS -i \"$SenderAddr\""
	[ -n "$AETitle" ] && INPUT_PARAMS="$INPUT_PARAMS -a \"$AETitle\""
fi


# storescp_reg.sh can be called directly:
# run as a task instead
if [ `basename $0` = "storescp_reg.sh" ] ; then
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Deprecated call: ARGS: $@"
	echo "WARNING: storescp_reg.sh is deprecated. Use StorescpReg class instead:" >&2
	echo "  direct call:" >&2
	echo "      $TASKD_HOME/runjavacmd -c \"0 cases.StorescpReg " "$@" "\"" >&2
	echo "  or create and schedule a task:" >&2
	echo "     $TASKD_HOME/createtaskfileapp -p \"STORESCP_REG\" -t \"class\" -c \"cases.StorescpReg $@\"" >&2
	$TASKD_HOME/runjavacmd -c "0 cases.StorescpReg $INPUT_PARAMS"
	exit $? 
fi

$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "ARGS: $*"

run_() # override run_ method
{
	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "$TASKD_HOME/runjavacmd -n -c \"0 cases.StorescpReg $INPUT_PARAMS\""
	$TASKD_HOME/runjavacmd -c "0 cases.StorescpReg $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
	ERR=$?

	exit $ERR
}


# override cleanup_
cleanup_()
{
	$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "$TASKD_HOME/runjavacmd -c \"cleanup cases.StorescpReg $INPUT_PARAMS\""
	$TASKD_HOME/runjavacmd -c "cleanup cases.StorescpReg $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
}


# override print_
print_()
{
	$TASKD_HOME/runjavacmd -c "print cases.StorescpReg $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
}


# override get_watson_message_
get_watson_message_()
{
	[ -z "$DR_WATSON_ID" ] && DR_WATSON_ID="storescpreg.$Study"
	[ -z "$DR_WATSON_MESSAGE" ] && DR_WATSON_MESSAGE="Can't pre-register $Study"

	if [ "$RUN_RESULT" -gt 124 ] ; then
		WATSON_MESSAGE_ID="$DR_WATSON_ID.failed"
		WATSON_MESSAGE_TEXT="FAILED: $DR_WATSON_MESSAGE"
		return
	fi

	if [ "$RUN_RESULT" -ne 0 -a "$RETRY_NUM" -gt 0 -a $(( $RETRY_NUM % 10)) -eq 0 ] ; then
		WATSON_MESSAGE_TEXT="ERROR: $DR_WATSON_MESSAGE [Retry #$RETRY_NUM]"
		WATSON_MESSAGE_ID="$DR_WATSON_ID.$RETRY_NUM"
	fi
} # get_watson_message_()
