<%@
	page errorPage="../main/exceptionHandler.jsp"

%><% throw new Exception("Unimplemented JSP");%>
<%--
 * Component: checkintegrity
 *
 * Description:
 *   If the device is az archive device, than call compareobjinfo.sh.
 *   The result is 0 than OK wil return. Otherwise ERROR is the message.
 *
 * Params:
 *   cStudy             -  Study Instance UID of the study to be checked
 *   cObject            -  Object file name to be checked.
 *   cFile              -  Object info file was generated on caller device
 *   cIgnoreEditChanges -  Ignore edit changes based on the PbR content in the archive
 *                         Soft edit changes will be ignored when comparing non PbR objects, but
 *                         PbRs will be compared as is.
 *   cDebug             -  Print debug messages
 *
-- %><%@

	page errorPage="../main/exceptionHandler.jsp"

%><%@
	page import="java.io.*, java.sql.*, java.util.*,
			com.erad.pacs.services.usermanager.client.Envelope,
			com.erad.pacs.services.usermanager.client.Session,
			com.erad.pacs.services.usermanager.tomcat.EnvelopeManager,
			com.erad.pacs.connector.Connector,
			com.erad.pacs.services.usermanager.client.interfaces.UserManagerFacade,
			imagemed.imsession.*, imagemed.checkintegrity.*, imagemed.util.*,
			imagemed.dotcom.*, imagemed.jsputil.DotcomToolbox, imagemed.repohandler.*"

%><%!

	/**
	 * Prints OK result (meaning the object is archived correctly) to writer in a processable format
	 *
	 * @param out   writer object to print on
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void printOK(JspWriter out) throws IOException {
		out.println("#RESULT: OK");
	}

	/**
	 * Prints OK-FILTER result (meaning the study does not match to archive filter and so should not be archived) to writer in a processable format
	 *
	 * @param out   writer object to print on
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void printOKFilter(JspWriter out) throws IOException {
		out.println("#RESULT: OK-FILTER");
	}


	/**
	 * Prints Error result (meaning an error occured while checking the object) to writer in a processable format
	 *
	 * @param out      writer object to print on
	 * @param message  error message to be printed
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void printError(JspWriter out, String message) throws IOException {
		out.println("#RESULT: ERROR");
		out.println("#MESSAGE: " + message);
	}

	/**
	 * Prints Failed result (the object is not correctly archived) to writer in a processable format
	 *
	 * @param out      writer object to print on
	 * @param code     error code (STUDY_NOT_IN_DB, OBJECT_DIFFERS, ...)
	 * @param message  error message to be printed
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void printFailed(JspWriter out, String code, String message) throws IOException {
		out.println("#RESULT: FAILED");
		out.println("#CODE: " + code);
		out.println("#MESSAGE: " + message);
	}

	/**
	 * Prints Pending result (meaning study wasn't registered) to writer in a processable format
	 *
	 * @param out           writer object to print on
	 * @param filterFields  ID of fields used in archive filters (empty list means archive filter newver was set)
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void printPending( JspWriter out, Set<String> filterFields ) throws IOException {
		out.println( "#RESULT: PENDING" );
		
		if ( ! filterFields.isEmpty() ) {
			out.print( "#MESSAGE:" );
			for ( Iterator<String> iter = filterFields.iterator(); iter.hasNext(); ) {
				out.print( " " );
				out.print( iter.next() );
			}
			out.println();
		}
	}

	
	/**
	 * Compares the canonized object to the object in archive/study directory.
	 *
	 * @param out                writer object to print on
	 * @param file               the canonized object received in the HTTP request
	 * @param studyDir           location of the object to be compared
	 * @param object             file name of the object to be checked (relative to studyDir)
	 * @param ignoreEditChanges  ignore edit changes when comparing the object
	 * @param isDebug            print debug messages
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void compareObjects(JspWriter out, String file, String studyDir, String object, boolean ignoreEditChanges, boolean isDebug) throws IOException {
		String cmd[] = new String[ignoreEditChanges ? 8 : 7]; 

		cmd[0] = IMEnvBase.getEnv("CHECKINTEGRITY_HOME") + "/compareobjinfo.sh";
		cmd[1] = "-c";
		cmd[2] = file;
		cmd[3] = "-d";
		cmd[4] = studyDir;
		cmd[5] = "-o";
		cmd[6] = object;
		if (ignoreEditChanges)
			cmd[7] = "-i";

		RunCommandResult res = Helper.runCommandEx(cmd, null);

		if (isDebug) {
			StringBuffer logCmd = new StringBuffer();
			for (int i = 0; i < cmd.length ; i++) {
				logCmd.append(" ").append(cmd[i]);
			}
			out.println("DEBUG cmd: " + logCmd);
		}

		int exitCode = res.getExitCode();
		if (exitCode == 0) {
			printOK(out);
			return;
		}

		if (exitCode == 11) {
			printFailed(out, "OBJECT_NOT_IN_DIR", "Object does not exist in archive directory");
			return;
		}

		if (20 < exitCode && exitCode < 30) {
			// 21 - Differences in number of files extracted
			// 22 - Differences in dcmdump files
			// 23 - Differences in pixel data
			printFailed(out, "OBJECT_DIFFERS", "Object differs from the archived one [" + exitCode + "]");
			return;
		}

		printError(out,"An error occured while comparing objects");

		if (isDebug) {
			out.println("DEBUG: Error Code: " + exitCode);
			out.println("DEBUG: Error Stream: " + res.getError());
		}
	}

	/**
	 * Checks if study/object exists in the external archive
	 *
	 * @param out                writer object to print on
	 * @param study              study instance uid to be checked
	 * @param object             object to be checked
	 * @param isDebug            print debug messages
	 *
	 * @throws IOException if I/O error occurs
	 */
	static void checkInExternalArchive(JspWriter out, String study, String object, boolean isDebug) throws IOException {
		// The current device is the peer - call 127.0.0.1
		String cmd[] = new String[3];
		cmd[0] = IMEnvBase.getEnv("CHECKINTEGRITY_HOME") + "/checkExternalArchive.sh";
		cmd[1] = study;
		cmd[2] = object;
		
		RunCommandResult res = Helper.runCommandEx(cmd, new String[] {"HOME=" + IMEnvBase.getEnv("HOME")});

		if (isDebug) {
			StringBuffer logCmd = new StringBuffer();
			for (int i = 0; i < cmd.length ; i++) {
				logCmd.append(" ").append(cmd[i]);
			}
			out.println("DEBUG cmd: " + logCmd);
		}

		int exitCode = res.getExitCode();
		if (exitCode == 0) {
			printOK(out);
			return;
		}

		if (exitCode > 1) {
			printError(out, "An error occured while checking object in external archive");
			if (isDebug) {
				out.println("DEBUG: Error Code: " + exitCode);
				out.println("DEBUG: Error Stream: " + res.getError());
			}
			return;
		}

		printFailed(out, "OBJECT_NOT_IN_DIR", "Object does not exist in archive directory");
		if (isDebug) {
			out.println("DEBUG: Error Code: " + exitCode);
			out.println("DEBUG: Error Stream: " + res.getError());
		}
	}

	static void checkArchiveList(Session session, JspWriter out, String study, boolean isDebug) throws SQLException, IOException {
		ArchiveListChecker alc = new ArchiveListChecker();
		if ( isDebug ) {
			alc.setDebugOut( out );
		}
		
		ArchiveListChecker.ErrorCode err = alc.load();

		if ( err != ArchiveListChecker.ErrorCode.OK ) {
			printError( out,  err.getErrorMessage() );
			return;
		}

		ArchiveListChecker.CheckResult res = alc.checkInDB( session, study, ArchiveListChecker.Mode.MATCH_ANY_SINCE_DATE );
		if ( res == ArchiveListChecker.CheckResult.STUDY_DOES_NOT_MATCH ) {
			// Study should not be archived
			printOKFilter( out );
			return;
		}
		
		if ( res == ArchiveListChecker.CheckResult.STUDY_NOT_REGISTERED ) {
			printPending( out, alc.getFilterFields() );
			return;
		}
		
		printFailed( out, res.getName(), res.getMessage() );	
	}		
	

	/**
	 *
	 * @return true if request was issued to check against static data rather than study in the database
	 */
	static boolean checkArchiveListsOnly( Session session, JspWriter out, MultipartRequest mr ) throws IOException, SQLException {
		String numFields = mr.getParameter( "cFieldsNum" );
		if ( numFields == null )
			return false;
		
		int nFields = Integer.parseInt( numFields );
		TreeMap<String, String> fieldValues = new TreeMap<String, String>(); 
		for ( int i = 0; i < nFields; i++ ) {
			String fld = mr.getParameter( "cField_" + i );
			String value = mr.getParameter( "cValue_" + i );
			
			fieldValues.put( fld, value );
		}

		boolean isDebug = "true".equals( mr.getParameter( "cDebug" ) );
		
		ArchiveListChecker alc = new ArchiveListChecker();
		if ( isDebug ) {
			alc.setDebugOut( out );
		}

		ArchiveListChecker.ErrorCode err = alc.load();

		if ( err != ArchiveListChecker.ErrorCode.OK ) {
			printError( out,  err.getErrorMessage() );
			return true;
		}

		
		ArchiveListChecker.CheckResult res = alc.checkStaticData( session, fieldValues, ArchiveListChecker.Mode.MATCH_ANY_SINCE_DATE );
		if ( res == ArchiveListChecker.CheckResult.STUDY_DOES_NOT_MATCH ) {
			// Study should not be archived
			printOKFilter( out );
			return true;
		}
		
		printFailed( out, res.getName(), res.getMessage() );	
		return true;
	}

	
%><%
	response.setContentType("text/plain");


	String peerId = null;
	try {
		peerId = DotcomToolbox.checkClient( request.getRemoteAddr() ).getProperty( "ID" );
	}
	catch (Throwable th) {
		if ( !"127.0.0.1".equals( request.getRemoteAddr() ) ) {
			printError(out, "Access denied");
			return;
		}
	}

	Envelope envelope = Connector.getInstance().get( UserManagerFacade.Caller ).loginService( SessionInfo.SYSTEM_USERID );
	Session	sess = EnvelopeManager.getInstance( request, response ).checkForRmiSubstitute( envelope );

	// This page can also be called from self device, but in this case peerId will be null

	MultipartRequest mr = new MultipartRequest(request, IMEnvBase.getEnv("TMP"));

	if ( checkArchiveListsOnly( sess, out, mr ) ) {
		// checkInArchive.jsp was called to check if study data matches to archive filters
		return;
	}
	
	
	String parStudy = mr.getParameter("cStudy");
	String parObject = mr.getParameter("cObject");
	String parFile = mr.getParameter("cFile");
	boolean ignoreEditChanges = "true".equals(mr.getParameter("cIgnoreEditChanges"));
	boolean isDebug = "true".equals(mr.getParameter("cDebug"));

	if (parStudy == null) {
		printError(out, "Missing parameter: cStudy");
		return;
	}

	if (parObject == null) {
		printError(out, "Missing parameter: cObject");
		return;
	}

	if (parFile == null) {
		printError(out, "Missing parameter: cFile");
		return;
	}

	if (isDebug)
		out.println("DEBUG: " + parFile);

	File param = new File(parFile);
	if (!param.exists() || param.length() == 0) {
		printError(out, parFile + " does not exist or has zero size");
		return;
	}

	if (isDebug)
		out.println("DEBUG: param file size :" + param.length());

	
	// Check if the object exists in data/dicom and compare with it instead of the one in the archive
	// This is for avoiding to overwrite a newer object by resending an older one in the cleanup
	// Differing objects are intended to be fixed manually
	if (peerId != null) {
		String studyDir = RepositoryHandler.getLocation(Repository.DICOM, parStudy);
		if (studyDir != null && (new File(studyDir + "/" + parObject)).exists()) {
			compareObjects(out, parFile, studyDir, parObject, ignoreEditChanges, isDebug);
			return;
		}
	}
		
	DotcomDevice selfDevice = DotcomConfig.selfDevice();

	StoreState storestate = new StoreState(parStudy);
	storestate.load();
	String nextToTop = storestate.getNextToStudyTop(selfDevice);

	if (nextToTop != null) {
		if (isDebug)
			out.println("DEBUG: Pass check in archive request to " + nextToTop);
		
		String host = null;
			
		ResultSet rs = IMEnv.getDB().prepareStatement("select HOST from " + IMEnvBase.getEnv("TARGETTBL") + " where ID='" + nextToTop + "'").executeQuery();
		if (rs.next()) {
			 host = rs.getString("HOST");
		}
		rs.close();

		if (host == null) {
			printError(out, "No information on " + nextToTop + " found in the Target table");
			return;
		}

		if (isDebug)
			out.println("DEBUG: send request to " + nextToTop );

		String result = null;
		
		try {
			Dotcom.getMultipartHttpPage( nextToTop, "/checkintegrity/checkInArchive.jsp", mr, null, null );
		}
		catch ( Exception ex ) {
			printError(out, "An error occured while sending request to " + nextToTop + "[" + ex.toString() + "]" );
			return;
		}
		
		if (result.indexOf("#RESULT: ") == -1) {
			if (isDebug) {
				out.println("DEBUG: response received from " + nextToTop + ":");
				out.println(result);
			}
			printError(out, "Invalid response recevied from " + nextToTop);
			return;
		}

		if (isDebug)
			out.println("DEBUG: response received from " + nextToTop + " seems to be valid");

		out.print(result);
		return;
	}

	Properties archivePrp = selfDevice.getArchiveSettings();
	String archiveType = archivePrp == null ? null : archivePrp.getProperty("Archive");

	if ("yes".equals(archiveType) || "readonly".equals(archiveType)) {
		// archive is connected to the current server - check object in the archive
		if (isDebug)
			out.println("DEBUG: start checking object file");
			
		String archivePath = IMEnvBase.getEnv("ARCHIVE_DIR") + "/";

		ResultSet rs = IMEnv.getDB().prepareStatement("select ARCHDIR from " + IMEnvBase.getEnv("ARCHIVETBL") + 
				" where STYIUID='" + parStudy + "'").executeQuery();
		if (rs.next()) {
				archivePath += rs.getString("ARCHDIR") + "/" + parStudy;
		}
		else {
			checkArchiveList( sess, out, parStudy, isDebug );
			return;
		}
		rs.close();

		if (isDebug)
			out.println("DEBUG: study archive directory: " + archivePath);

		if (!(new File(archivePath + "/" + parObject)).exists()) {
			printFailed(out, "OBJECT_NOT_IN_DIR", "Object does not exist in archive directory");
			return;
		}

		// Compare the object in the archive
		compareObjects(out, parFile, archivePath, parObject, ignoreEditChanges, isDebug);
	}
	else if ("external".equals(archiveType)) {
		checkInExternalArchive(out, parStudy, parObject, isDebug);
	}
	else {
		printFailed(out, "NO_ARCHIVE_ON_SERVER", "No archive configured on " + selfDevice.getId());
		return;
	}

--%>
