#!/bin/sh
#
# Component: dotcom
#
# Description:
#   Applies proliferated directory.
#
# Usage:
#    applystoredir.sh -f <filename> -t <targetdir> [-n] [-d <direction>] [-x <excepthost>]
#
#      -f <filename>   : name of the stored (tar) file (proliferate)
#      -t <targetdir>  : target to untar <filename>
#      -n              : do not proliferate to Database servers
#      -x <excepthost> : don't send the file to this host (Host ID (preferred) / IP address (obsolete))
#      -d <direction>  : direction of proliferation {up|down|all}, default is all
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="applystore[$$]"

#
# Usage
#
usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<- ENDOFUSAGE
		applystoredir.sh -f <filename> -t <targetdir> [-n] [-d <direction>] [-x <excepthost>]
	
		 -f <filename>   : name of the stored (tar) file (proliferate)
		 -t <targetdir>  : target to untar <filename>
		 -n              : do not proliferate to Database servers
		 -x <excepthost> : don't send the file to this host (Host ID (preferred) / IP address (obsolete))
		 -d <direction>  : direction of proliferation {up|down|all}, default is all
	ENDOFUSAGE

	[ -n "$1" ] && exit 1
	exit 0
} # usage


ERRMSG=""
FILENAME=""
PDIRECTION=""
PEXCEPT=""
TARGETDIR=""

EXCLUDE_DBONLY=""

while getopts 't:f:x:d:n' OPT; do
	case $OPT in
		f) FILENAME="$OPTARG" ;;
		t) TARGETDIR="$OPTARG" ;;
		x) PEXCEPT="-x \"$OPTARG\"" ;;
		d) PDIRECTION="-d \"$OPTARG\"" ;;
		n) EXCLUDE_DBONLY="-n" ;;
		*) usage_ "Invalid_Parameter:$OPT" ;;
	esac
done

[ -z "$FILENAME" -o -z "$TARGETDIR" ] && usage_ "Missing Parameter"

[ ! -f "$FILENAME" -o ! -d "$TARGETDIR" ] && exit 1

CWD=`pwd`
cd "$TARGETDIR"
tar xf "$FILENAME"
cd "$CWD"

$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "call proliferate.sh $FILENAME"

$TASKD_HOME/createtaskfileapp -p "PROLIFERATE" -t "sh" -c "$DOTCOM_HOME/proliferate.sh -f \"$FILENAME\" $EXCLUDE_DBONLY $PDIRECTION $PEXCEPT -r \"$DOTCOM_HOME/applystoredir.sh\" -a -t -a \"$TARGETDIR\""

exit 0
